﻿using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class SymbolicExpressionGrammarValueConfiguration : ParameterizedValueConfiguration {

    #region Constructors and Cloning
    [StorableConstructor]
    protected SymbolicExpressionGrammarValueConfiguration(bool deserializing) : base(deserializing) { }
    public SymbolicExpressionGrammarValueConfiguration() : base() { }
    public SymbolicExpressionGrammarValueConfiguration(ISymbolicExpressionGrammar grammar) {
      this.IsOptimizable = true;
      this.ActualValue = new ConstrainedValue(grammar, grammar.GetType(), new ItemSet<IItem> { grammar }, false);
    }
    protected SymbolicExpressionGrammarValueConfiguration(SymbolicExpressionGrammarValueConfiguration original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new SymbolicExpressionGrammarValueConfiguration(this, cloner);
    }
    #endregion

    protected override void PopulateParameterConfigurations(IItem item, bool discoverValidValues) {
      var grammar = (ISymbolicExpressionGrammar)item;
      foreach (Symbol symbol in grammar.Symbols) {
        this.parameterConfigurations.Add(new SingleValuedParameterConfiguration(symbol.Name, new SymbolValueConfiguration(symbol)));
      }
    }

    public virtual void Parameterize(ISymbolicExpressionGrammar grammar) {
      foreach (Symbol symbol in grammar.Symbols) {
        ((SymbolValueConfiguration)this.parameterConfigurations.Single(pc => pc.Name == symbol.Name).ValueConfigurations.First()).Parameterize(symbol);
      }
    }
  }
}
